@@ -63,6 +63,16 @@ class AttrDict(dict):
 config = AttrDict()
 
 
+def get_out_file_name(in_path, in_file):
+    count = 1
+    out_file = '%s.mp4' % in_file
+    while os.path.exists(os.path.join(in_path, out_file)):
+        out_file = '%s__2mp4_%s__.mp4' % (in_file, count)
+        count += 1
+
+    return out_file
+
+
 def get_media_info(file):
     xmlIO = StringIO()
     sh.mediainfo(
@@ -75,8 +85,8 @@ def get_media_info(file):
     tracks = []
 
     for track in info.tracks:
-        type = track.track_type.lower()
-        if type == 'general':
+        track_type = track.track_type.lower()
+        if track_type == 'general':
             general_info = track
         else:
             tracks.append(track)
@@ -94,6 +104,11 @@ def get_media_info(file):
     for index, track in enumerate(tracks):
         track.track_id = track_ids[index]
 
+    folder_name, file_name = os.path.split(file)
+    filename, ext = os.path.splitext(file_name)
+    general_info.folder_name = folder_name
+    general_info.file_name = filename
+
     return general_info, tracks
 
 
@@ -137,22 +152,26 @@ class EncodingProgress:
         self.pbar.finish()
 
 
-def get_video_opts(index, track):
-    if track.format == 'AVC':
+def get_video_opts(index, track, force_encode=False):
+    if track.format == 'AVC' and not force_encode:
         method = '1pass'
         video_opts = [
             '-map', '0:%s' % index,
             '-codec:v', 'copy'
         ]
     else:
-        method = '2pass'
         video_opts = [
             '-map', '0:%s' % index,
-            '-b:v', str(track.bit_rate),
             '-codec:v', 'libx264',
             '-profile:v', 'high',
             '-level', '4.1'
         ]
+        if track.bit_rate:
+            method = '2pass'
+            video_opts = video_opts + ['-b:v', track.bit_rate]
+        else:
+            method = '1pass'
+            video_opts = video_opts + ['-crf', '18']
 
     return method, video_opts
 
@@ -206,7 +225,11 @@ def convert(filename, args):
                 raise Exception(
                     "2mp4 currently doesn't support multiple video streams :("
                 )
-            method, video_opts = get_video_opts(track.track_id, track)
+            method, video_opts = get_video_opts(
+                track.track_id,
+                track,
+                force_encode=args.force_encode
+            )
             frame_count = track.frame_count
             if frame_count is None:
                 frame_count = float(
@@ -219,7 +242,10 @@ def convert(filename, args):
         elif track_type == 'text':
             subtitle_opts += get_subtitle_opts(track.track_id, track)
 
-    out_file_name = '%s.mp4' % general_info.file_name
+    out_file_name = get_out_file_name(
+        general_info.folder_name,
+        general_info.file_name
+    )
     out_path = os.path.join(general_info.folder_name, out_file_name)
     sys.stderr.write('Encoding %s -> %s\n' % (
         filename, out_file_name.encode('utf-8'))
@@ -229,6 +255,14 @@ def convert(filename, args):
         sys.stderr.write('Destination file exists, skipping...\n')
         return
 
+    # Test that we can write to output path
+    try:
+        with open(out_path, 'wb'):
+            os.unlink(out_path)
+    except IOError as e:
+        print e
+        return sys.exit(e.errno)
+
     if method == '1pass':
         opts = input_ops + video_opts + audio_opts + \
             subtitle_opts + metadata_opts + [
@@ -367,6 +401,16 @@ def main():
             "just show the command(s) that would be executed"
         )
     )
+    parser.add_argument(
+        '-f', '--force-encode',
+        dest='force_encode',
+        action='store_true',
+        default=False,
+        help=(
+            "Force a re-encode of the video stream, even if it is already "
+            "in a format supported by mp4 container."
+        )
+    )
     parser.add_argument(
         '-v', '--version',
         action='version',
